// File: Fw_HbmRuler.ino
// Firmware module for readout of 2 HBM rulers for a lathe
// Version 2015-10-23  by jan@breem.nl
// Target: Arduino Nano 
// Pin Use:
// Pin 5, PD2 is Clock to both rulers
// Pin 23, PC4 is Data from the Z-ruler
// Pin 24, PC5 is Data from the X-ruler
// Pin 11, PB0 indicates the Pause signal
// Pin 12, PB1 indicates the ISR in service time and frequency
// Pin 13, PB2 Output for loudspeaker;

#include <arduino.h>

long RulerX, RulerZ;  // store the raw values from the rulers
byte BeepTime;

//********************************************************************************************************
ISR(TIMER2_COMPA_vect)  // called at 20 kHz = 50 usec interval
// Service takes between 2 and 5 usec.
{ 
  static byte TimeSlice;
  static bool Pause, SndWave;
  static long TempRulerX, TempRulerZ; 
  static byte SoundCnt;
  
  PORTB |= 0x02; // Diagnose total service time of ISR
  
  // Process sound
  if (SoundCnt > 0) SoundCnt--; else 
  {
    SoundCnt = 10; // frequency 1000 Hz
    SndWave = !SndWave; 
    if (BeepTime > 0) // number of period to beep
    {
      if (SndWave) PORTB |= 0x04; else PORTB &= ~0x04;
      BeepTime--; 
    }
    else PORTB &= ~0x04; // shut off speaker
  }
  
  // Readout of rulers
  switch (TimeSlice++ & 0x03) // modulo 4
  {
  case 0:  // set clock high
    if (!Pause) PORTD |= 0x04; // clock high
    break; 

  case 1: // sample and clock low
    if (!Pause)
      if ((PINC & 0x20) == 0x20) TempRulerX |= 0x80000000; // Set MSB bit
        else TempRulerX &= ~0x80000000; // Clear MSB bit of result
      if ((PINC & 0x10) == 0x10) TempRulerZ |= 0x80000000; // Set MSB bit
        else TempRulerZ &= ~0x80000000; // Clear MSB bit of result
      // Clearing the MSB is nescessary because of sign extend while shifting
    PORTD &= ~0x04; // clock low
    break;
    
  case 2: 
    if (!Pause) 
    {
      TempRulerX = TempRulerX >> 1; // shift right. Rulers give LSB first
      TempRulerZ = TempRulerZ >> 1; // shift right  
    }
    break;
    
  case 3: 
    if (TimeSlice == 84) // 21 cycles of 4 steps
    {
      RulerX = TempRulerX >> 10; // Final shift to get LSB right
      RulerZ = TempRulerZ >> 10;
      Pause = true; 
      PORTB |= 0x01;  // Diagnose start of Pause
    }
    if (TimeSlice == 200) 
    {
      TimeSlice = 0;  
      TempRulerX = 0;
      TempRulerZ = 0;
      Pause = false;  // Ftart again
      PORTB &= ~0x01; // Diagnose end of Pause
    }
    break;
  }
  // if you need to do time consuming things do it while Pause == true
  // or in timeslices 84 < TS < 200
  // so missing some interrupts will not harm the ruler readout
  
  PORTB &= ~0x02;  // Diagnose ISR in service time
} // ISR(TIMER2_COMPA_vect)

//********************************************************************************************************
void init_HbmRuler(void)
{
  // Configure 8 bit Timer2 for readout timing
  TCCR2A = 0x02;  // WaveformMode 2, no PWM outputs
  TCCR2B = 0x02;  // internal clock div 8 for 2 MHz
  TIMSK2 = 0x02;  // interrupt on OCRA match   
  OCR2A = 99;     // divide by 100 for 20 kHz interrupts
  // Data Direction bits 
  DDRD |= 0x04;   // clock output on pin 5
  DDRB |= 0x01;   // diagnostic pin 11, Pause
  DDRB |= 0x02;   // diagnostic pin 12, Interrupt In Sercvice
  DDRB |= 0x04;   // output for loudspeaker on pin 13
}

